-- 
-- Specialization IndoorAnimation
-- 
-- @author  	Aluha74rus
-- @edit:		OverClocker
-- @history:	v2.0 - 2020-01-12 - edited for t150 05-09 by OverClocker
--
IndoorAnimation = {};

local IndoorAnimation_directory = g_currentModDirectory;

function IndoorAnimation.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(PowerTakeOffs, specializations)
end;

function IndoorAnimation.registerEventListeners(vehicleType)		
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", IndoorAnimation);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", IndoorAnimation)
	SpecializationUtil.registerEventListener(vehicleType, "onUpdateTick", IndoorAnimation)
end

function IndoorAnimation:onLoad(savegame)
    self.KnopkaSignal = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.enterable.dashboards.keys.keyHonk#animationName"), "keyHonk")
	
    local klawishaPoworotNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.enterable.dashboards.keys.turnLightKey#node"), self.i3dMappings);
    if klawishaPoworotNode ~= nil then
        self.klawishaPoworot = {};
        self.klawishaPoworot.node = klawishaPoworotNode;
        local x, y, z = StringUtil.getVectorFromString(getXMLString(self.xmlFile, "vehicle.enterable.dashboards.keys.turnLightKey#minRot"));
        self.klawishaPoworot.minRot = {};
        self.klawishaPoworot.minRot[1] = MathUtil.degToRad(Utils.getNoNil(x, 0));
        self.klawishaPoworot.minRot[2] = MathUtil.degToRad(Utils.getNoNil(y, 0));
        self.klawishaPoworot.minRot[3] = MathUtil.degToRad(Utils.getNoNil(z, 0));

        local x, y, z = StringUtil.getVectorFromString(getXMLString(self.xmlFile, "vehicle.enterable.dashboards.keys.turnLightKey#rotLeft"));
        self.klawishaPoworot.leftRot = {};
        self.klawishaPoworot.leftRot[1] = MathUtil.degToRad(Utils.getNoNil(x, 0));
        self.klawishaPoworot.leftRot[2] = MathUtil.degToRad(Utils.getNoNil(y, 0));
        self.klawishaPoworot.leftRot[3] = MathUtil.degToRad(Utils.getNoNil(z, 0));

        local x, y, z = StringUtil.getVectorFromString(getXMLString(self.xmlFile, "vehicle.enterable.dashboards.keys.turnLightKey#rotRight"));
        self.klawishaPoworot.rightRot = {};
        self.klawishaPoworot.rightRot[1] = MathUtil.degToRad(Utils.getNoNil(x, 0));
        self.klawishaPoworot.rightRot[2] = MathUtil.degToRad(Utils.getNoNil(y, 0));
        self.klawishaPoworot.rightRot[3] = MathUtil.degToRad(Utils.getNoNil(z, 0));

        self.klawishaPoworot.rotTime = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.enterable.dashboards.keys.turnLightKey#rotTime"), 1)*1000;
		self.klawishaPoworot.lastDir = 1;
    end;
    self.KlawishaSvet = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.enterable.dashboards.keys.lightKeyFront#node"), self.i3dMappings);  
    self.KlawishaSvetD = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.enterable.dashboards.keys.workLightBackKey#node"), self.i3dMappings); 
	
end
function IndoorAnimation:onUpdate(dt, isActiveForInput, isSelected)
	if Input.isKeyPressed(Input.KEY_0)then
		if self.KnopkaSignal ~= nil and self.playAnimation ~= nil then
            self:playAnimation(self.KnopkaSignal, 1, nil, true);
		end;
	else
		if not self.KnopkaSignal ~= nil and self.playAnimation ~= nil then
            self:playAnimation(self.KnopkaSignal, -1, nil, true);
		end;
	end
end;

function IndoorAnimation:onUpdateTick(dt, isActiveForInput, isSelected)
    local spec = self.spec_motorized
    local spec = self.spec_lights
    if self:getIsActive() then
        if self.spec_lights.turnLightState ~= nil then
			if self.klawishaPoworot ~= nil then
				local dir = 0;
				local hasRotChanged = false;
				local destRot = self.klawishaPoworot.minRot;
				
				if self.spec_lights.turnLightState == 1 then
					destRot = self.klawishaPoworot.leftRot;
					dir = 1;
				elseif self.spec_lights.turnLightState == 2 then
					destRot = self.klawishaPoworot.rightRot;
					dir = -1;
				end
				if self.klawishaPoworot.lastDir ~= dir then
					self.klawishaPoworot.lastDir = dir;
					self.klawishaPoworot.rotatingTime = 0;
				end
				
				if self.klawishaPoworot.rotatingTime ~= nil then
					self.klawishaPoworot.rotatingTime = self.klawishaPoworot.rotatingTime + dt;
					local t = math.min(1.0, self.klawishaPoworot.rotatingTime / self.klawishaPoworot.rotTime);
					if t == 1 then
						self.klawishaPoworot.rotatingTime = nil;
					end
					local rot = {getRotation(self.klawishaPoworot.node)};
					
					local rx1,ry1,rz1,rw1 = mathEulerToQuaternion(unpack(rot));
					local rx2,ry2,rz2,rw2 = mathEulerToQuaternion(unpack(destRot));
					
					local qx,qy,qz,qw = MathUtil.nlerpQuaternionShortestPath(rx1,ry1,rz1,rw1, rx2,ry2,rz2,rw2, t);
					setQuaternion(self.klawishaPoworot.node, qx,qy,qz,qw);
				end
			end;
		end;
        if lightsTypesMask ~= spec.lightsTypesMask or force then
            if (spec.lightsTypesMask == 1)then
                setTranslation(self.KlawishaSvet, -0, -0.009, 0);
            elseif (spec.lightsTypesMask > 2)then
                setTranslation(self.KlawishaSvet, -0, 0.002, 0);
            else
                setTranslation(self.KlawishaSvet, -0, -0.017, 0);
            end;
            if (spec.lightsTypesMask > 3)then
                setRotation(self.KlawishaSvetD, MathUtil.degToRad(-45), 0, 0);
            else
                setRotation(self.KlawishaSvetD, MathUtil.degToRad(0), 0, 0);
            end;
	    end;
	end;
end